@@ -8,10 +8,11 @@ use Catalyst::Action;
 use Catalyst::ActionContainer;
 use Catalyst::DispatchType::Default;
 use Catalyst::DispatchType::Index;
+use Catalyst::Utils;
 use Text::SimpleTable;
 use Tree::Simple;
 use Tree::Simple::Visitor::FindByPath;
-use Scalar::Util ();
+use Scalar::Util qw(blessed);
 
 # Stringify to class
 use overload '""' => sub { return ref shift }, fallback => 1;
@@ -127,6 +128,116 @@ sub dispatch {
     }
 }
 
+# $self->_command2action( $c, $command [, \@arguments ] )
+# $self->_command2action( $c, $command [, \@captures, \@arguments ] )
+# Search for an action, from the command and returns C<($action, $args, $captures)> on
+# success. Returns C<(0)> on error.
+
+sub _command2action {
+    my ( $self, $c, $command, @extra_params ) = @_;
+
+    unless ($command) {
+        $c->log->debug('Nothing to go to') if $c->debug;
+        return 0;
+    }
+
+    my (@args, @captures);
+    
+    if ( ref( $extra_params[-2] ) eq 'ARRAY' ) {
+        @captures = @{ pop @extra_params };
+    }
+
+    if ( ref( $extra_params[-1] ) eq 'ARRAY' ) {
+        @args = @{ pop @extra_params }
+    } else {
+        # this is a copy, it may take some abuse from
+        # ->_invoke_as_path if the path had trailing parts
+        @args = @{ $c->request->arguments };
+    }
+
+    my $action;
+
+    if (Scalar::Util::blessed($command) && $command->isa('Catalyst::Action')) {
+        $action = $command;
+    }
+    else {
+        # go to a string path ("/foo/bar/gorch")
+        # or action object which stringifies to that
+        $action = $self->_invoke_as_path( $c, "$command", \@args );
+    }
+
+    # go to a component ( "MyApp::*::Foo" or $c->component("...")
+    # - a path or an object)
+    unless ($action) {
+        my $method = @extra_params ? $extra_params[0] : "process";
+        $action = $self->_invoke_as_component( $c, $command, $method );
+    }
+
+    return $action, \@args, \@captures;
+}
+
+=head2 $self->visit( $c, $command [, \@arguments ] )
+
+Documented in L<Catalyst>
+
+=cut
+
+sub visit {
+    my $self = shift;
+    $self->_do_visit('visit', @_);
+}
+
+sub _do_visit {
+    my $self = shift;
+    my $opname = shift;
+    my ( $c, $command ) = @_;
+    my ( $action, $args, $captures ) = $self->_command2action(@_);
+    my $error = qq/Couldn't $opname("$command"): /;
+
+    if (!$action) {
+        $error .= qq/Couldn't $opname to command "$command": /
+                 .qq/Invalid action or component./;
+    }
+    elsif (!defined $action->namespace) {
+        $error .= qq/Action has no namespace: cannot $opname() to a plain /
+                 .qq/method or component, must be an :Action of some sort./
+    }
+    elsif (!$action->class->can('_DISPATCH')) {
+        $error .= qq/Action cannot _DISPATCH. /
+                 .qq/Did you try to $opname() a non-controller action?/;
+    }
+    else {
+        $error = q();
+    }
+
+    if($error) {
+        $c->error($error);
+        $c->log->debug($error) if $c->debug;
+        return 0;
+    }
+
+    $action = $self->expand_action($action);
+
+    local $c->request->{arguments} = $args;
+    local $c->request->{captures}  = $captures;
+    local $c->{namespace} = $action->{'namespace'};
+    local $c->{action} = $action;
+
+    $self->dispatch($c);
+}
+
+=head2 $self->go( $c, $command [, \@arguments ] )
+
+Documented in L<Catalyst>
+
+=cut
+
+sub go {
+    my $self = shift;
+    $self->_do_visit('go', @_);
+    die $Catalyst::GO;
+}
+
 =head2 $self->forward( $c, $command [, \@arguments ] )
 
 Documented in L<Catalyst>
@@ -134,33 +245,9 @@ Documented in L<Catalyst>
 =cut
 
 sub forward {
-    my ( $self, $c, $command, @extra_params ) = @_;
-
-    unless ($command) {
-        $c->log->debug('Nothing to forward to') if $c->debug;
-        return 0;
-    }
-
-    my @args;
-    
-    if ( ref( $extra_params[-1] ) eq 'ARRAY' ) {
-        @args = @{ pop @extra_params }
-    } else {
-        # this is a copy, it may take some abuse from ->_invoke_as_path if the path had trailing parts
-        @args = @{ $c->request->arguments };
-    }
-
-    my $action;
-
-    # forward to a string path ("/foo/bar/gorch") or action object which stringifies to that
-    $action = $self->_invoke_as_path( $c, "$command", \@args );
-
-    # forward to a component ( "MyApp::*::Foo" or $c->component("...") - a path or an object)
-    unless ($action) {
-        my $method = @extra_params ? $extra_params[0] : "process";
-        $action = $self->_invoke_as_component( $c, $command, $method );
-    }
-
+    my $self = shift;
+    my ( $c, $command ) = @_;
+    my ( $action, $args, $captures ) = $self->_command2action(@_);
 
     unless ($action) {
         my $error =
@@ -171,9 +258,7 @@ sub forward {
         return 0;
     }
 
-    #push @$args, @_;
-
-    local $c->request->{arguments} = \@args;
+    local $c->request->{arguments} = $args;
     $action->dispatch( $c );
 
     return $c->state;
@@ -213,35 +298,44 @@ sub _invoke_as_path {
     }
 }
 
-sub _find_component_class {
+sub _find_component {
     my ( $self, $c, $component ) = @_;
 
-    return ref($component)
-      || ref( $c->component($component) )
-      || $c->component($component);
+    # fugly, why doesn't ->component('MyApp') work?
+    return $c if ($component eq blessed($c));
+
+    return blessed($component)
+        ? $component
+        : $c->component($component);
 }
 
 sub _invoke_as_component {
-    my ( $self, $c, $component, $method ) = @_;
+    my ( $self, $c, $component_or_class, $method ) = @_;
 
-    my $class = $self->_find_component_class( $c, $component ) || return 0;
+    my $component = $self->_find_component($c, $component_or_class);
+    my $component_class = blessed $component || return 0;
 
-    if ( my $code = $class->can($method) ) {
+    if (my $code = $component_class->can('action_for')) {
+        my $possible_action = $component->$code($method);
+        return $possible_action if $possible_action;
+    }
+
+    if ( my $code = $component_class->can($method) ) {
         return $self->method_action_class->new(
             {
                 name      => $method,
                 code      => $code,
-                reverse   => "$class->$method",
-                class     => $class,
+                reverse   => "$component_class->$method",
+                class     => $component_class,
                 namespace => Catalyst::Utils::class2prefix(
-                    $class, $c->config->{case_sensitive}
+                    $component_class, $c->config->{case_sensitive}
                 ),
             }
         );
     }
     else {
         my $error =
-          qq/Couldn't forward to "$class". Does not implement "$method"/;
+          qq/Couldn't forward to "$component_class". Does not implement "$method"/;
         $c->error($error);
         $c->log->debug($error)
           if $c->debug;
@@ -380,6 +474,25 @@ sub uri_for_action {
     return undef;
 }
 
+=head2 expand_action 
+
+expand an action into a full representation of the dispatch.
+mostly useful for chained, other actions will just return a
+single action.
+
+=cut
+
+sub expand_action {
+    my ($self, $action) = @_;
+
+    foreach my $dispatch_type (@{ $self->dispatch_types }) {
+        my $expanded = $dispatch_type->expand_action($action);
+        return $expanded if $expanded;
+    }
+
+    return $action;
+}
+
 =head2 $self->register( $c, $action )
 
 Make sure all required dispatch types for this action are loaded, then
@@ -473,11 +586,15 @@ sub setup_actions {
     $self->_load_dispatch_types( @{ $self->postload_dispatch_types } );
 
     return unless $c->debug;
+    $self->_display_action_tables($c);
+}
 
+sub _display_action_tables {
+    my ($self, $c) = @_;
+
+    my $column_width = Catalyst::Utils::term_width() - 20 - 36 - 12;
     my $privates = Text::SimpleTable->new(
-        [ 20, 'Private' ],
-        [ 36, 'Class' ],
-        [ 12, 'Method' ]
+        [ 20, 'Private' ], [ 36, 'Class' ], [ $column_width, 'Method' ]
     );
 
     my $has_private = 0;
@@ -527,10 +644,30 @@ sub _load_dispatch_types {
     return @loaded;
 }
 
-=head1 AUTHOR
+=head2 $self->dispatch_type( $type )
 
-Sebastian Riedel, C<sri@cpan.org>
-Matt S Trout, C<mst@shadowcatsystems.co.uk>
+Get the DispatchType object of the relevant type, i.e. passing C<$type> of
+C<Chained> would return a L<Catalyst::DispatchType::Chained> object (assuming
+of course it's being used.) 
+
+=cut
+
+sub dispatch_type {
+    my ($self, $name) = @_;
+
+    unless ($name =~ s/^\+//) {
+        $name = "Catalyst::DispatchType::" . $name;
+    }
+
+    for (@{ $self->dispatch_types }) {
+        return $_ if ref($_) eq $name;
+    }
+    return undef;
+}
+
+=head1 AUTHORS
+
+Catalyst Contributors, see Catalyst.pm
 
 =head1 COPYRIGHT
 
