@@ -18,8 +18,9 @@ Catalyst::Test - Test Catalyst Applications
 
     # Tests
     use Catalyst::Test 'TestApp';
-    request('index.html');
-    get('index.html');
+    my $content  = get('index.html');           # Content as string
+    my $response = request('index.html');       # HTTP::Response object
+    my($res, $c) = ctx_request('index.html');   # HTTP::Response & context object
 
     use HTTP::Request::Common;
     my $response = request POST '/foo', [
@@ -61,7 +62,7 @@ object.
 
 =head2 METHODS
 
-=head2 get
+=head2 $content = get( ... )
 
 Returns the content.
 
@@ -78,12 +79,19 @@ method and the L<request> method below:
     is ( $uri->path , '/y');
     my $content = get($uri->path);
 
-=head2 request
+=head2 $res = request( ... );
 
 Returns a C<HTTP::Response> object.
 
     my $res = request('foo/bar?test=1');
 
+=head1 FUNCTIONS
+
+=head2 ($res, $c) = ctx_request( ... );
+
+Works exactly like C<Catalyst::Test::request>, except it also returns the
+catalyst context object, C<$c>. Note that this only works for local requests.
+
 =cut
 
 sub import {
@@ -110,11 +118,40 @@ sub import {
 
     no strict 'refs';
     my $caller = caller(0);
-    *{"$caller\::request"} = $request;
-    *{"$caller\::get"}     = $get;
+    
+    *{"$caller\::request"}      = $request;
+    *{"$caller\::get"}          = $get;
+    *{"$caller\::ctx_request"}  = sub { 
+        my $me      = ref $self || $self;
+
+        ### throw an exception if ctx_request is being used against a remote
+        ### server
+        Catalyst::Exception->throw("$me only works with local requests, not remote")     
+            if $ENV{CATALYST_SERVER};
+    
+        ### place holder for $c after the request finishes; reset every time
+        ### requests are done.
+        my $c;
+        
+        ### hook into 'dispatch' -- the function gets called after all plugins
+        ### have done their work, and it's an easy place to capture $c.
+        no warnings 'redefine';
+        my $dispatch = Catalyst->can('dispatch');
+        local *Catalyst::dispatch = sub {
+            $c = shift;
+            $dispatch->( $c, @_ );
+        };
+        
+        ### do the request; C::T::request will know about the class name, and
+        ### we've already stopped it from doing remote requests above.
+        my $res = $request->( @_ );
+        
+        ### return both values
+        return ( $res, $c );
+    };
 }
 
-=head2 local_request
+=head2 $res = Catalyst::Test::local_request( $AppClass, $url );
 
 Simulate a request using L<HTTP::Request::AsCGI>.
 
@@ -135,7 +172,7 @@ sub local_request {
 
 my $agent;
 
-=head2 remote_request
+=head2 $res = Catalyst::Test::remote_request( $url );
 
 Do an actual remote request using LWP.
 
